from django.test import TestCase
from django.test.utils import override_settings
from django_skebby.utils import Sms, skebby_credit_left, SkebbySmsError, SkebbySendError


class TestSkebby(TestCase):
    def test_basic_sms(self):
        sms = Sms("Hi there!", ["123456789"], sender_string="Your Friend")
        self.assertEqual(sms.headers, {'user-agent': 'Generic Client'})

    def test_template_rendering(self):
        sms = Sms("Happy {{ festivity }}!", ["123456789"], sender_string="Your friend", ctx={'festivity': 'birthday'})
        self.assertEqual(sms.text, "Happy birthday!")

    def test_invalid_sender_num_string(self):
        ret = False
        try:
            sms = Sms("Hi there!", ["123456789"], sender_string="Your friend", sender_number="123456789")
        except SkebbySmsError:
            ret = True
        self.assertEqual(ret, True)

    def test_invalid_sender_string(self):
        ret = False
        try:
            sms = Sms("Hi there!", ["123456789"], sender_string="Longer than 11 chars")
        except SkebbySmsError:
            ret = True
        self.assertEqual(ret, True)

    def test_invalid_sender_string_basic_method(self):
        ret = False
        sms = Sms("Hi there!", ["123456789"], sender_string="Me")
        try:
            sms.send(method="basic")
        except SkebbySendError:
            ret = True
        self.assertEqual(ret, True)

    def test_invalid_sender_number_basic_method(self):
        ret = False
        sms = Sms("Hi there!", ["123456789"], sender_number="39123456789")
        try:
            sms.send(method="basic")
        except SkebbySendError:
            ret = True
        self.assertEqual(ret, True)

    def test_invalid_charset(self):
        ret = False
        try:
            sms = Sms("Hi there!", ["123456789"], charset="foo",  sender_number="123456789")
        except SkebbySmsError:
            ret = True
        self.assertEqual(ret, True)

    def test_credit_left(self):
        r = skebby_credit_left()
        credit = r.skebby_response
        self.assertEqual(True, 'credit_left' in credit)
        self.assertEqual(True, 'classic_sms' in credit)
        self.assertEqual(True, 'basic_sms' in credit)

    def test_send(self):
        sms = Sms("Hi there!", ["39123456789"], sender_string="Your Friend")
        ret = sms.send(method="test")
        failed_requests = [r for r in ret if r.skebby_error]
        succesful_requests = [r for r in ret if not r.skebby_error]
        self.assertEqual(len(failed_requests) + len(succesful_requests), 1)

    def test_send_single(self):
        sms = Sms("Hi {{ friend }}!", sender_string="Your Friend")
        ret = sms.send_single({'friend': 'Doge'}, "39123456789", method="test")
        self.assertEqual(sms.text, "Hi Doge!")
        self.assertEqual(False, ret.skebby_error)
        self.assertEqual("", ret.skebby_message)

    @override_settings(SKEBBY_DEFAULT_METHOD='foobar')
    def test_default_invalid_method(self):
        ret = False
        sms = Sms("Hi there!", ["123456789"])
        try:
            sms.send()
        except SkebbySendError:
            ret = True
        self.assertEqual(ret, True)



from setuptools import setup, find_packages
import os
import django_skebby

CLASSIFIERS = [
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
]

setup(
    author="Riccardo Magliocchetti",
    author_email="riccardo.magliocchetti@gmail.com",
    name='django-skebby',
    version=django_skebby.__version__,
    description='A Django app to send sms with skebby',
    long_description=open(os.path.join(os.path.dirname(__file__), 'README.md')).read(),
    url="https://github.com/xrmx/django-skebby",
    license='BSD License',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    install_requires=[
        'Django>=1.4',
        'requests>=2',
    ],
    packages=find_packages(exclude=["example", "example.*"]),
    include_package_data=True,
    zip_safe=False,
)



from django.conf import settings
from django.utils.encoding import smart_str
from django.template.loader import get_template_from_string
from django.template.context import Context
import requests

try:
    SKEBBY_URL = settings.SKEBBY_URL
except AttributeError:
    SKEBBY_URL = "https://gateway.skebby.it/api/send/smseasy/advanced/http.php"

try:
    SKEBBY_VERIFY_SSL_CERTIFICATE = settings.SKEBBY_VERIFY_SSL_CERTIFICATE
except AttributeError:
    SKEBBY_VERIFY_SSL_CERTIFICATE = True

SKEBBY_METHODS = {
    'basic': 'send_sms_basic',
    'classic': 'send_sms_classic',
    'report': 'send_sms_classic_report',
    'test': 'test_send_sms_classic',
}

SKEBBY_CHARSETS = ('ISO-8859-1', 'UTF-8')

DEFAULT_MAX_RECIPIENTS = 50000
# this must be agreed with skebby
MAX_RECIPIENTS = 100000


class SkebbySmsError(Exception):
    pass


class SkebbySendError(Exception):
    pass


def _parse_response(response):
    try:
        response.raise_for_status()
        text = response.text.split('&')
        result = {}
        for pair in text:
            k, v = pair.split('=')
            result[k] = v
        response.skebby_response = result

        if result['status'] == 'failed':
            response.skebby_error_ = True
            response.skebby_message = result['message']
        else:
            response.skebby_error = False
            response.skebby_message = ""
    except requests.exceptions.HTTPError as e:
        response.skebby_response = {}
        response.skebby_message = e
        response.skebby_error = True
    return response


class Sms:
    def __init__(self, text, recipients=None, sender_number=None, sender_string=None, charset=None, ctx=None, headers=None):
        self.template = get_template_from_string(text)
        self.ctx = {} if ctx is None else ctx
        self.text = smart_str(self.template.render(Context(self.ctx)))

        if sender_number and sender_string:
            raise SkebbySmsError("Only one between sender_number and sender_string may be specified")

        if sender_string and len(sender_string) > 11:
            raise SkebbySmsError("sender_string too long, 11 chars max")

        self.sender_number = sender_number
        self.sender_string = sender_string

        self.charset = "UTF-8" if charset is None else charset
        if self.charset not in SKEBBY_CHARSETS:
            raise SkebbySmsError("Invalid charset")

        self.headers = {'user-agent': 'Generic Client'} if headers is None else headers
        self.recipients = [] if recipients is None else recipients

        try:
            self.max_recipients = settings.SKEBBY_MAX_RECIPIENTS
        except AttributeError:
            self.max_recipients = DEFAULT_MAX_RECIPIENTS

        if self.max_recipients > MAX_RECIPIENTS:
            self.max_recipients = MAX_RECIPIENTS

    def _check_method(self, method):
        if not method:
            try:
                method = settings.SKEBBY_DEFAULT_METHOD
            except AttributeError:
                method = "classic"

        if method not in SKEBBY_METHODS:
            raise SkebbySendError("Invalid send method")

        # method specific checks
        if method == "basic":
            if self.sender_number:
                raise SkebbySendError("sender_number not supported with basic method")
            elif self.sender_string:
                raise SkebbySendError("sender_string not supported with basic method")

        return SKEBBY_METHODS.get(method)

    def send(self, method=None):
        username = settings.SKEBBY_USERNAME
        password = settings.SKEBBY_PASSWORD

        method = self._check_method(method)

        num_remainders = len(self.recipients)
        if not num_remainders:
            raise SkebbySendError("No recipients")

        ret = []
        while num_remainders:
            remainders = self.recipients[:self.max_recipients]
            payload = {
                'recipients': remainders,
                'method': method,
                'username': username,
                'password': password,
                'text': self.text,
                'sender_number': self.sender_number,
                'sender_string': self.sender_string,
                'charset': self.charset,
            }
            r = requests.post(SKEBBY_URL, data=payload, headers=self.headers, verify=SKEBBY_VERIFY_SSL_CERTIFICATE)
            num_remainders -= len(remainders)
            ret.append(_parse_response(r))
        return ret

    def send_single(self, ctx, recipient, method=None):
        username = settings.SKEBBY_USERNAME
        password = settings.SKEBBY_PASSWORD

        method = self._check_method(method)
        self.text = smart_str(self.template.render(Context(ctx)))

        payload = {
            'recipients': recipient,
            'method': method,
            'username': username,
            'password': password,
            'text': self.text,
            'sender_number': self.sender_number,
            'sender_string': self.sender_string,
            'charset': self.charset,
        }
        r = requests.post(SKEBBY_URL, data=payload, headers=self.headers, verify=SKEBBY_VERIFY_SSL_CERTIFICATE)
        return _parse_response(r)


def skebby_credit_left():

    username = settings.SKEBBY_USERNAME
    password = settings.SKEBBY_PASSWORD

    payload = {
        'method': 'get_credit',
        'username': username,
        'password': password,
    }

    r = requests.post(SKEBBY_URL, data=payload, verify=SKEBBY_VERIFY_SSL_CERTIFICATE)
    return _parse_response(r)



from django.http import HttpResponse
from django_skebby.utils import skebby_credit_left
import json


def credit_left(request):
    credit = skebby_credit_left()
    credit_dict = credit.skebby_response if not credit.skebby_error else {}
    return HttpResponse(json.dumps(credit_dict), mimetype="application/json")
